/*
* $LastChangedBy$
* $LastChangedDate$
* $LastChangedRevision$
* $HeadURL$
*
* Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
*/

import Foundation
/**
Interface Connection
*/
@objc public class Connection: NSObject {
    private var _id: String
    private var _parameters = [String: [String]]()

    @objc init(id: String, parameters: [String: [String]]) {
        self._id = id
        self._parameters = parameters
    }

    /**
    Returns the id of the connection.
    &lt;pre&gt:
    // Swift:
    let context: InteractionContext!
    let connection: Connection? = context.getConnection(connectionId)
    let connectionId = connection.getId()

    // Objective-C:
    InteractionContext* context;
    Connection* connection = [context getConnection:connectionId];
    NSString* connectionId = [connection getId];
    &lt;/pre&gt:

    - returns:     The id of the connection.
    */
    @objc public func getId() -> String {
        return self._id
    }

    /**
    Returns the parameters of the connection.
    &lt;pre&gt:
    // Swift:
    let context: InteractionContext!
    let connection: Connection? = context.getConnection(connectionId)
    let connectionParameters: Dictionary&lt;String, [String]&gt; = connection.getParameters()

    // Objective-C:
    InteractionContext* context;
    Connection* connection = [context getConnection:connectionId];
    NSDictionary* connectionParameters = [connection getParameters];
    &lt;/pre&gt:

    - returns:     The parameters of the connection.
    */
    @objc public func getParameters() -> [String: [String]] {
        return self._parameters
    }

}
