/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */

public class EngagementScoreListener: BlueConicPlugin {

    private var _client: BlueConicClient!
    private var _context: InteractionContext!
    private var _engagementService: EngagementService?

    /**
     Default initializer for a blueconic plugin
     - parameter client: An instance of the BlueConicClient
     - parameter context: An instance of the InteractionContext
     */
    public required convenience init(client: BlueConicClient, context: InteractionContext) {
        self.init()
        self._client = client
        self._context = context
    }

    /**
     OnLoad function for blueconic plugin.
     This starts the plugin.
     */
    public func onLoad() {
        guard let interactionId = self._context.getInteractionId(),
            let engagementRules = ListenerUtil.getDictionaryFromParameters(self._context, key: PluginConstants.Listener.TAG_ENGAGEMENT_RULES)
            else {
                return
        }

        guard let profileProperty = self.getProfilePropertyFromRules(self._context) else {
            return
        }

        BlueConicEventFactory.getInstance().cleanup()
        self._engagementService = EngagementService(client: self._client, listenerId: interactionId, propertyName: profileProperty, isInterest: false, allInterests: [], useHalfTime: false)
        guard let service = self._engagementService else {
            return
        }

        if let rules = engagementRules[PluginConstants.Listener.TAG_RULES] {
            service.applyRules(rules)
        }
        
        // Check if there are changes in the engagementservice
        if service.isChanged() {
            // Store the internal JSON string or the changes to the profile
            service.save()
        }
    }

    /**
     Return the profile property from the listener rules
     Based on the new or old format, it will retrieve the correct profileProperty
     This starts the plugin.
     */
    private func getProfilePropertyFromRules(_ context: InteractionContext) -> String? {
        var profileProperty: String?

        if let propertyRules = ListenerUtil.getArrayFromParameters(context, key: PluginConstants.Listener.TAG_PROPERTY_RULES) {
            // > v.6.2.0
            var highDecay = 0
            for rule in propertyRules {
                if let ruleProfileProperty = rule[PluginConstants.Listener.TAG_PROFILE_PROPERTY] as? String,
                    let decay = rule[PluginConstants.Listener.TAG_DECAY] as? Int,
                    decay > highDecay {
                    highDecay = decay
                    profileProperty = ruleProfileProperty
                }
            }
        } else {
            // < v6.2.0
            profileProperty = ListenerUtil.getProfilePropertyFromParameters(context.getParameters())
        }

        return profileProperty
    }

    public func onDestroy() {
        BlueConicEventFactory.getInstance().clearEventHandlers(self._context.getInteractionId())
    }

}
