/*
 * $LastChangedBy$
 * $LastChangedDate$
 * $LastChangedRevision$
 * $HeadURL$
 *
 * Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
 */

public class FormListener: BlueConicPlugin {
    private var _client: BlueConicClient!
    private var _context: InteractionContext!

    /**
     Default initializer for a blueconic plugin
     - parameter client: An instance of the BlueConicClient
     - parameter context: An instance of the InteractionContext
     */
    public convenience required init(client: BlueConicClient, context: InteractionContext) {
        self.init()
        self._client = client
        self._context = context
    }

    /**
     OnLoad function for blueconic plugin.
     This starts the plugin.
     */

    public func onLoad() {
        BlueConicEventFactory.getInstance().cleanup()
        if let rules = self.migrate(),
            let interactionId = self._context.getInteractionId() {
            let formService = FormListeningService(self._client, listenerId: interactionId)
            formService.applyRules(rules)
            formService.save()
        }
    }

    /**
     Migration code for this the form listener
     */
    public func migrate() -> [Any]? {
        let parameters = self._context.getParameters()
        
        if let version = parameters[PluginConstants.Listener.TAG_VERSION]?[0],
            let listenerRules = parameters[PluginConstants.Listener.TAG_FORM_LISTENER_RULES] {

            if version == "2" {
                // already at the latest version, no need for migration
                return ListenerUtil.stringToJson(listenerRules[0]) as? [Any]
            } else if let listenerRules = ListenerUtil.getDictionaryFromParameters(_context, key: PluginConstants.Listener.TAG_FORM_LISTENER_RULES),
                let rules = listenerRules["rules"] {
                // migrate from version 1 to version 2
                return migrateToV2(rules: rules)
            }
        }
        return nil
    }

    /**
     Migration code from version 1 to version 2
     */
    public func migrateToV2(rules: [Any]) -> [Any]? {
        return rules.map { (rule: Any) -> Any in

            // define a new rule and the old rule to convert the old data to the new version 2
            var newRule = rule as? [String: Any]
            let oldRule = newRule

            // Migrate the identifier to id
            newRule?[PluginConstants.Listener.MAPPINGS.ID] = oldRule?[PluginConstants.Listener.MAPPINGS.OLD.ID]
            
            // Migrate the structure of the profile property parameter
            if let profileProperties = oldRule?[PluginConstants.Listener.TAG_PROFILE_PROPERTY] as? [[String: Any]] {
                newRule?[PluginConstants.Listener.TAG_FORM_PROFILE_PROPERTY] = profileProperties[0][PluginConstants.Listener.TAG_PROFILE_PROPERTY] ?? ""
            }

            if var converter = oldRule?[PluginConstants.Listener.TAG_CONVERTER] as? [String: Any],
                var mappings = converter[PluginConstants.Listener.TAG_MAPPINGS] as? [[String: Any]] {

                mappings = mappings.map { (mapping: [String: Any]) -> [String: Any] in
                    var newMapping = mapping
                    // Update mappings to the V2 format [identifier -> id]
                    newMapping[PluginConstants.Listener.MAPPINGS.ID] =
                        newMapping[PluginConstants.Listener.MAPPINGS.OLD.ID] ??
                        newMapping[PluginConstants.Listener.MAPPINGS.ID]

                    // Update mappings to the V2 format [o -> original]
                    newMapping[PluginConstants.Listener.MAPPINGS.ORIGINAL] = newMapping[PluginConstants.Listener.MAPPINGS.OLD.ORIGINAL] ?? newMapping[PluginConstants.Listener.MAPPINGS.ORIGINAL]

                    // Update mappings to the V2 format [c -> converted]
                    newMapping[PluginConstants.Listener.MAPPINGS.CONVERTED] = newMapping[PluginConstants.Listener.MAPPINGS.OLD.CONVERTED] ?? newMapping[PluginConstants.Listener.MAPPINGS.CONVERTED]

                    return newMapping
                }

                converter[PluginConstants.Listener.TAG_MAPPINGS] = mappings
                newRule?[PluginConstants.Listener.TAG_CONVERTER] = converter
            }

            return newRule as Any
        }
    }

    public func onDestroy() {
        BlueConicEventFactory.getInstance().clearEventHandlers(self._context.getInteractionId())
    }

}
